/* oe_memory.c -- For user-supplied memory allocation functions

   Copyright (C) 2005 Robin Hogan <r.j.hogan@reading.ac.uk> 

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <string.h>
#include <stdlib.h>

#include "oe.h"

/* Set the oe-library memory allocation/deallocation functions to the
   standard libc ones */
void (*oe_free) (void *) = free;
void* (*oe_malloc) (size_t size) = malloc;
void* (*oe_realloc) (void *ptr, size_t nbytes) = realloc;

/* Set the oe-library memory allocation/deallocation functions to the
   ones specified; e.g. to use the default libc ones, you could use
   oe_set_memory_functions(malloc, realloc, free); */
void
oe_set_memory_functions(void* (*malloc_function) (size_t),
			void* (*realloc_function) (void *, size_t),
			void (*free_function) (void *))
{
  oe_malloc = malloc_function;
  oe_realloc = realloc_function;
  oe_free = free_function;
}

/* Behaves the same as libc's strdup() but uses oe_malloc() to
   allocate memory instead of malloc() */
char *
oe_strdup(const char *s)
{
  int len = strlen(s);
  char *dup;

  dup = oe_malloc(sizeof(char) * (len+1));
  if (!dup) {
    return NULL;
  }
  else {
    int i = 0;
    do {
      dup[i] = s[i];
    } while(s[i++]);
    return dup;
  }
}
