#include <fstream>
#include <cmath>
#include "rplot.h"
#define PI_OVER_180 0.01745329
#define DEBUG cerr << "mapplot.c: reached line " << __LINE__ << endl;

float *coast_x, *coast_y;
int coast_n = 0;

float *location_x, *location_y;
int location_n = 0;
string *location_str;

float longitude, latitude;


int
MAPSetOrigin(float lat, float lon)
{
	longitude = lon;
	latitude = lat;
	return 0;
}

int
MAPLoadCoastline(char *mapfile)
{
	ifstream ifs(mapfile);
	int i;

	if (!ifs.is_open()) {
		return 1;
	}
	ifs >> coast_n;
	if (coast_n < 1) {
		ifs.close();
		coast_n = 0;
		return 2;
	}
	if ((coast_x = new float[coast_n]) == NULL) {
		ifs.close();
		coast_n = 0;
		return 3;
	}
	if ((coast_y = new float[coast_n]) == NULL) {
		ifs.close();
		coast_n = 0;
		return 3;
	}

	for (i=0; i<coast_n; i++) {
		ifs >> coast_x[i] >> coast_y[i];
		coast_x[i] = (coast_x[i] - longitude)*110*cos(coast_y[i]*PI_OVER_180);
		coast_y[i] = (coast_y[i] - latitude)*110;
	}
	ifs.close();
	return 0;
}

int
MAPFreeCoastline()
{
	if (coast_n > 0) {
		delete[] coast_x;
		delete[] coast_y;
		return 0;
	}
	else {
		return 1;
	}
}

int
MAPLoadLocations(char *locfile)
{
	ifstream ifs(locfile);
	int i;
	if (!ifs.is_open()) {
		return 1;
	}
	ifs >> location_n;
	if (location_n < 1) {
		ifs.close();
		location_n = 0;
		return 2;
	}
	if ((location_x = new float[location_n]) == NULL) {
		ifs.close();
		location_n = 0;
		return 3;
	}
	if ((location_y = new float[location_n]) == NULL) {
		ifs.close();
		location_n = 0;
		return 3;
	}

	if ((location_str = new string[location_n]) == NULL) {
		ifs.close();
		location_n = 0;
		return 3;
	}

	for (i=0; i<location_n; i++) {
		ifs >> location_str[i] >> location_x[i] >> location_y[i];
	}
	ifs.close();
	return 0;
}

int
MAPFreeLocations()
{
	if (location_n > 0) {
		delete[] location_x;
		delete[] location_y;
		delete[] location_str;
		return 0;
	}
	else {
		return 1;
	}
}

int
MAPPlotCoastline(RAxes *axes)
{
	axes->plot(coast_x, coast_y, coast_n);
	return 0;
}

int
MAPPlotLocations(RAxes *axes)
{
	int i;

	for (i=0; i<location_n; i++) {
		if (axes->axis().in(location_x[i],location_y[i])) {
			axes->plot(location_x[i]-1,location_y[i],location_x[i]+1,location_y[i]);
			axes->plot(location_x[i],location_y[i]-1,location_x[i],location_y[i]+1);
			axes->text(location_x[i]+1,location_y[i],location_str[i].c_str(),ANCHOR_LEFT);
		}
	}
	return 0;
}
