#include <stdio.h>
#include "zdr_phidp.h"

int
load_scattering(char *filename, ScatteringData *scattering)
{
  rc_data *data = rc_read(filename, stderr);

  if (!data) {
    OE_THROW(OE_FAILURE, "Scattering file not found");
  }
  OE_CHECK(oe_read_matrix(data, "ZH", &scattering->Zh));
  OE_CHECK(oe_read_matrix(data, "ZV", &scattering->Zv));
  OE_CHECK(oe_read_matrix(data, "attenuationH", &scattering->attenh));
  OE_CHECK(oe_read_matrix(data, "attenuationV", &scattering->attenv));
  OE_CHECK(oe_read_matrix(data, "Kdp", &scattering->Kdp));
  OE_CHECK(oe_read_vector(data, "diameter", &scattering->diameter));
  OE_CHECK(oe_read_vector(data, "temperature", &scattering->temperature));
  OE_CHECK(oe_read_vector(data, "K2", &scattering->K2));

  rc_clear(data);
  return OE_SUCCESS;
}

void
free_scattering(ScatteringData *scattering)
{
  oe_free_matrix(&scattering->Zh);
  oe_free_matrix(&scattering->Zv);
  oe_free_matrix(&scattering->attenh);
  oe_free_matrix(&scattering->attenv);
  oe_free_matrix(&scattering->Kdp);
  oe_free_vector(&scattering->diameter);
  oe_free_vector(&scattering->temperature);
  oe_free_vector(&scattering->K2);
}
