#include <stdio.h>
#include "zdr_phidp.h"
#include "readconfig.h"

#define CHECK(status, name, type) if (!status) { \
   fprintf(stderr, "Error reading variable \"%s\" as %s\n", \
   name, type); return OE_FAILURE; }

#define REAL_NAME "double"

#define GET_REAL(var, name) \
   var = rc_get_real(config, name, &status); \
   CHECK(status, name, REAL_NAME);

#define GET_INT(var, name) \
   var = rc_get_real(config, name, &status); \
   CHECK(status, name, "int");


int
load_settings(int argc, char **argv, Settings *settings)
{
  rc_data *config;
  int ifile = rc_get_file(argc, argv);
  int status;

  if (!ifile) {
    /* No file given - assume command-line arguments contain all the
       information. */
    config = rc_read(NULL, stderr);
  }
  else {
    /* Read configuration information from the file. */
    config = rc_read(argv[ifile], stderr);
  }
  if (!config) {
    fprintf(stderr, "Error initializing configuration information\n");
    exit(1);
  }
  
  /* Supplement configuration information with command-line
     arguments. */
  rc_register_args(config, argc, argv);
  settings->argc = argc;
  settings->argv = argv;

  CHECK((settings->input = rc_get_string(config, "input")),
	"input", "string");
  CHECK((settings->lut_filename = rc_get_string(config,
	"lookup_table_file")), "lookup_table_file", "string");

  /* Settings with defaults */
  rc_assign_int(config, "distribution_num",
		&settings->distribution_num_D0);
  rc_assign_real(config, "distribution_min_D0",
		 &settings->distribution_min_D0);
  rc_assign_real(config, "distribution_interval_D0",
		 &settings->distribution_interval_D0);

  /* Setting that must be provided */
  GET_REAL(settings->distribution_mu, "distribution_shape_factor");
  GET_REAL(settings->a_default, "ZR_coefficient_apriori");
  GET_REAL(settings->b, "ZR_exponent");
  GET_REAL(settings->var_ln_a_apriori, "ln_a_var_apriori");
  settings->prehail_var_ln_a_apriori = settings->var_ln_a_apriori;
  GET_INT(settings->n_basis, "num_basis");

  GET_REAL(settings->azim_decorr_dist, "azimuth_decorrelation_distance");
  GET_REAL(settings->radial_decorr_dist, "radial_decorrelation_distance");

  /* Hail */
  settings->do_hail_retrieval = rc_get_boolean(config, "retrieve_hail");
  settings->save_first_pass = rc_get_boolean(config, "save_first_pass");

  if (settings->do_hail_retrieval) {
    GET_REAL(settings->Zdr_prehail_error_factor, "Zdr_prehail_error_factor");
    GET_REAL(settings->min_hail_Zh, "hail_min_Zh");
    GET_REAL(settings->min_hail_Zdr_diff, "hail_min_Zdr_error");
    GET_INT(settings->min_consecutive_hail_gates,
	    "hail_min_consecutive_gates");
    rc_assign_real(config, "prehail_var_ln_a_apriori",
		   &settings->prehail_var_ln_a_apriori);
  }
  
  /* Settings with defaults */
  rc_assign_real(config, "max_ln_a_excursion",
		 &settings->max_ln_a_excursion);
  rc_assign_real(config, "Zdr_error_factor", &settings->Zdr_error_factor);
  rc_assign_real(config, "phi_dp_error_factor",
		 &settings->phi_dp_error_factor);
  rc_assign_real(config, "attenuation_scaling_apriori",
		 &settings->atten_scaling);
  rc_assign_real(config, "attenuation_scaling_var_apriori",
		 &settings->var_atten_scaling);
  rc_assign_real(config, "max_attenuation", &settings->max_atten);
  rc_assign_real(config, "attenuation_scaling_decorrelation",
		 &settings->atten_scaling_decorr);
  rc_assign_string(config, "output_suffix", &settings->output_suffix);
  rc_assign_string(config, "output", &settings->output);
  rc_assign_string(config, "first_pass_output_suffix",
		   &settings->first_pass_output_suffix);
  rc_assign_string(config, "first_pass_output",
		   &settings->first_pass_output);

  rc_assign_int(config, "use_Zdr", &settings->use_Zdr);
  rc_assign_int(config, "use_phi_dp", &settings->use_phi_dp);

  /* Turn of Twomey smoothness constraint; use a-priori error
     covariance instead */
  settings->explicit_range_smoothing = 0;
  settings->confstring = rc_sprint(config);

  rc_print(config, stderr);
  rc_clear(config);
  return OE_SUCCESS;
}
