#include <math.h>
#include <stdio.h>
#include "zdr_phidp.h"
#include "rt.h"

#define NGATES 300

int
main(int argc, char **argv)
{
  ScatteringData scattering;
  ScatteringLUT lut;
  Settings settings = DEFAULT_SETTINGS;

  OEreal drange = 0.3;
  OEreal range[NGATES];
  OEreal temperature[NGATES];
  OEreal Zh_lin[NGATES];
  OEreal ln_a[NGATES];
  OEreal hf[NGATES];
  OEreal b = 1.5;

  OEreal Z = 40;
  OEreal a = 200;
  OEreal f = 0.0;

  OEreal Zdr[NGATES];
  OEreal phi_dp[NGATES];
  OEreal Ah[NGATES];
  OEreal Av[NGATES];

  OEreal dZdr_dlna_data[NGATES*NGATES];
  OEreal dphi_dp_dlna_data[NGATES*NGATES];
  OEreal dAh_dlna_data[NGATES*NGATES];
  OEreal dAv_dlna_data[NGATES*NGATES];

  
  OEreal **dZdr_dlna = rt_new_matrix(NGATES, NGATES);
  OEreal **dphi_dp_dlna = rt_new_matrix(NGATES, NGATES);
  OEreal **dAh_dlna = rt_new_matrix(NGATES, NGATES);
  OEreal **dAv_dlna = rt_new_matrix(NGATES, NGATES);
  OEreal **dZdr_dhf = rt_new_matrix(NGATES, NGATES);
  OEreal **dphi_dp_dhf = rt_new_matrix(NGATES, NGATES);
  OEreal **dAh_dhf = rt_new_matrix(NGATES, NGATES);
  OEreal **dAv_dhf = rt_new_matrix(NGATES, NGATES);

  int i;

  clear_struct(scattering);
  clear_struct(lut);

  OE_CATCH(load_scattering("scattering_goddard.cfg", &scattering));
  OE_CATCH(create_scattering_lut(&scattering,
				 settings.distribution_num_D0,
				 settings.distribution_min_D0,
				 settings.distribution_interval_D0,
				 settings.distribution_mu,
				 &lut));
  free_scattering(&scattering);

  if (argc > 1) {
    Z = pow(10.0, atof(argv[1]));
    if (argc > 2) {
      a = log(atof(argv[2]));
      if (argc > 3) {
	f = atof(argv[3]);
      }
    }
	    
  }


  for (i = 0; i < NGATES; i++) {
    range[i] = 0.15+i*0.3;
    temperature[i] = 10;
    Zh_lin[i] = 1.0;
    ln_a[i] = a;
    hf[i] = 0.0;
  }


  for (i = NGATES/2-10; i < NGATES/2+9; i++) {
    Zh_lin[i] = pow(10.0, 0.1*atof(argv[1]));
    hf[i] = f;
  }

  OE_CATCH(forward_model(&lut, NGATES, drange, range,
			 temperature, Zh_lin, ln_a, b,
			 hf, 1.0, 10.0,
			 /* Output */
			 Zdr, phi_dp, Ah, Av,
			 dZdr_dlna, dphi_dp_dlna, dAh_dlna, dAv_dlna,
			 dZdr_dhf, dphi_dp_dhf, dAh_dhf, dAv_dhf,
			 NULL, NULL, NULL, NULL));
  
  for (i = 0; i < NGATES; i++) {
    fprintf(stdout, "%g %g %g %g %g %g %g %g\n", range[i], temperature[i], 10.0*log10(Zh_lin[i]),
	    exp(ln_a[i]), Zdr[i], phi_dp[i], Ah[i], Av[i]);
  }

  exit(0);
}
